#include"LibNic.h"
#include"Symmetry.h"
#include"Green.h"
#include<iostream.h>

/*************************************************************************/
extern "C" {

//****Definition of the Green's tensor

void FortranCall(grtset)
      (int&     type, 
       Complex& kref,         //Wavenumber in the reference medium
       Complex& epsilonref,   //Permittivity of the reference medium
       double&  meshsize,     //Size of the mesh
       double&  theta);       //Theta (for type=2)

//type is 3   for 3D
//        2   for 2D
//        21  for 2DTM
//        22  for 2DTE
//       -2   for 2D   filtered
//       -3   for 3D   filtered
//       -21  for 2DTM filtered
//       -22  for 2DTE filtered

 
//****Evaluation of the Green's Tensor

void FortranCall(grtgel)
      (Tensor& T,             //Result
       vect3d& r,             //Position of observation
       vect3d& rp);            //Position of the source 


//****Evaluation of the magentic Green's Tensor

void FortranCall(grtgma)
      (Tensor& T,             //Result                 
       vect3d& r,	      //Position of observation
       vect3d& rp);           //Position of the source 


//****Evaluation of the source dyadic L

void FortranCall(grtl)(Tensor& T);              //Result


//****Evaluation of the self-induction term M

void FortranCall(grtm)(Tensor& T);              //Result


//****Evaluation of farfield Green's tensor

void FortranCall(grtgelff)
        (Tensor& T,           //Result                 
         vect3d& n,	      //Farfield direction
         vect3d& rp);	      //Position of the source 


//****Print the data

void FortranCall(grtprint)();

}

//**** Number type correspondence fortran - C++

//int:     integer
//double:  floating-point double precision
//Complex: complex double precision
//vect3d:  array of 3 double
//Tensor:  array of 9 Complex

//************************************************************************/

class GreenData{
 public:
  MeshGeometry MGeom;
  Complex      Kref;
  Complex      EpsilonRef;
  GreenTensor  *GT;
  int          Type;
  double       Theta;
 public:
  GreenData();
  ~GreenData();
  Print();
  void Test(){if(!Type) FatalError("Uninitialized Green's tensor");};
 public:
  Tensor GreenEl0(vect3d r,vect3d rp)   {Test(); return GT->GreenE0(r,rp);};
  Tensor GreenMa0(vect3d r,vect3d rp)   {Test(); return GT->GreenM0(r,rp);};
  Tensor GreenL()                       {Test(); return GT->L();};
  Tensor GreenM()                       {Test(); return GT->M();};
  Tensor GreenFFE0(vect3d n,vect3d rp)  {Test(); return GT->GreenFFE0(n,rp);};

  SetData(int type,Complex kref,Complex epsilonref, MeshGeometry mgeom, 
          double theta);
};

GreenData COMMONGT;

GreenData::GreenData()  

{ Type=0;
};

GreenData::~GreenData()  

{ if(Type!=0) delete GT;
  Type=0;
};

GreenData::SetData(int type,Complex kref,Complex epsilonref, 
                   MeshGeometry mgeom,double theta)

{ if(Type!=0) delete GT;
  Type=0;

  MGeom      =mgeom;
  Kref       =kref;
  EpsilonRef =epsilonref;
  Type       =type;
  Theta      =theta;

  double   kf  =PI/MGeom.x;
  Symmetry Symm=Symmetry(0,0,0);

  switch (Type)
    {
    case (3): 
      {
      EComponent EComp=EComponent(1,1,1);
      GT=new Green3d(Kref,EpsilonRef,MGeom,Symm,EComp);
      break;
      }
    case (-3): 
      {
      EComponent EComp=EComponent(1,1,1);
      GT=new FGreen3d(Kref,EpsilonRef,kf,MGeom,Symm,EComp);
      break;
      }
    case (2): 
      {
      EComponent EComp=EComponent(1,1,1);
      GT=new Green2d(Kref,EpsilonRef,MGeom,Symm,EComp,theta);
      break;
      }
    case (-2): 
      {
      EComponent EComp=EComponent(1,1,1);
      GT=new FGreen2d(Kref,EpsilonRef,kf,MGeom,Symm,EComp,theta);
      break;
      }
    case (21): 
      {
      EComponent EComp=EComponent(0,0,1);
      GT=new Green2dTM(Kref,EpsilonRef,MGeom,Symm,EComp);
      break;
      }
    case (-21): 
      {
      EComponent EComp=EComponent(0,0,1);
      GT=new FGreen2dTM(Kref,EpsilonRef,kf,MGeom,Symm,EComp);
      break;
      }
    case (22): 
      {
      EComponent EComp=EComponent(1,1,0);
      GT=new Green2dTE(Kref,EpsilonRef,MGeom,Symm,EComp);
      break;
      }
    case (-22): 
      {
      EComponent EComp=EComponent(1,1,0);
      GT=new FGreen2dTE(Kref,EpsilonRef,kf,MGeom,Symm,EComp);
      break;
      }
    default:
      FatalError("Green's tensor library: unexpected Green's tensor code");
      return -1;
    }
  return 0;
}
     

GreenData::Print()
 
{ cout<<"Type          "<<Type;
  switch (Type)
    {
    case (3):   cout<<"(3D)"; break;
    case (-3):  cout<<"(3D filtered)"; break;
    case (2):   cout<<"(2D (theta="<<Theta;"))"; break;
    case (-2):  cout<<"(2D filtered (theta="<<Theta;"))"; break;
    case (21):  cout<<"(2D TM)"; break;
    case (-21): cout<<"(2D TM filtered)"; break;
    case (22):  cout<<"(2D TE)"; break;
    case (-22): cout<<"(2D TE filtered)"; break;
    case (0):   cout<<"Not initialized!"; break;
    default:    cout<<"Invalide Green's tensor code!"; break;
    }
  cout<<"\n"; 
  cout<<"Mesh geometry "<<(Triplet<double>)MGeom<<"\n";
  cout<<"Kref          "<<Kref<<"\n";;
  cout<<"EpsilonRef    "<<EpsilonRef<<"\n";;

  flush(cout);
  return 0;
}

void FortranCall(grtset)(int&     type,
                         Complex& kref,
                         Complex& epsilonref,
                         double&  meshsize,
                         double&  theta)

{ MeshGeometry MG=MeshGeometry(meshsize,meshsize,meshsize);
  if(abs(type)!=3) MG.z=1;

  COMMONGT.SetData(type,kref,epsilonref,MG,theta);
}

void FortranCall(grtgel)(Tensor& T,vect3d& r,vect3d& rp)

{ T=!(COMMONGT.GT->GreenE0(r,rp));
}

void FortranCall(grtgma)(Tensor& T,vect3d& r,vect3d& rp)

{ T=!(COMMONGT.GT->GreenM0(r,rp));
}

void FortranCall(grtl)(Tensor& T)

{ T=!(COMMONGT.GT->L());
}

void FortranCall(grtm)(Tensor& T)

{ T=!(COMMONGT.GT->M());
}

void FortranCall(grtgelff)(Tensor& T,vect3d& n,vect3d& rp)

{ T=!(COMMONGT.GT->GreenFFE0(n,rp));
}

void FortranCall(grtprint)()

{ COMMONGT.Print();
}
