//**************************************************************************/
//*File Integrate.h
//*
//*Made the Aug 16 14:23 1995 by Nicolas Piller
//*
//*
//*Desctiption: Defininition of a classes to solve numericaly integrals.
//             
//*
//*Change: Date                       by
//*        ...                        ...
//**************************************************************************/
#if !defined(_INTEGRATE_)
#define _INTEGRATE_
 
#include <Complex.h>
#include <iostream.h>
#include "LibNic.h"

class IntegrationDatas{
 public:
  double AccRel;  
  int Pt;
  int Fcm;
  int Output;
 public:
  IntegrationDatas();
  IntegrationDatas(double accrel, int pt, int fcm);
  IntegrationDatas(double accrel, int pt, int fcm, int output);
  IntegrationDatas(const IntegrationDatas& id);
  PrintError(int OK);
};


class Integrate1d : public Object{ 
 private:
 protected:
  int IOK;
  int NFun;
  Object *OF;
  void (Object:: *MF)(double*,double);
  inline void FctValue(double*,double Var);
  double (*F)(double*,double*,Integrate1d*);
  double *TmpRes;
  friend double FUN(double* Var, double* Fc,Integrate1d* This);
  friend double FUNPInf(double* Var, double* Fc,Integrate1d* This);
  friend double FUNNInf(double* Var, double* Fc,Integrate1d* This);
  friend double FUNNPInf(double* Var, double* Fc,Integrate1d* This);
  double LimN, LimP;
  double RenN, RenP;
  int level;
 public:
  IntegrationDatas Datas;
  int M; 
  Integrate1d(int NFun);
  Integrate1d(int NFun,IntegrationDatas id);
  void SetLevel(int lev);
  ~Integrate1d();
  int Integrate(double* Res,Object *O,void (Object:: *FObj)(double*,double),
                double a,double b);
  int IntegratePInf(double* Res,Object *O,void(Object:: *FObj)(double*,double),
                    double a,double RenPos);
  int IntegrateNInf(double* Res,Object *O,void(Object:: *FObj)(double*,double),
                    double b,double RenNeg);
  int IntegrateNPInf(double* Res,Object *O,void(Object:: *FObj)(double*,double)
                    ,double RenNeg,double RenPos);
  int OK(){return IOK;};
};
  
class Integrate1dnd : public Integrate1d{
 private:
  int Level;
  double a, b;
  double *Var;
 protected:
 public:
  Integrate1dnd();
  Integrate1dnd(int NFun);
  void SetConst(int Level_, IntegrationDatas id, double *Var_);
  ~Integrate1dnd(){};
  void SetFunction(Object *O, void (Object:: *FObj)(double*,double));
  void SetLimits(double A, double B){a=A; b=B;};
  void IntegrateValue(double* V, double X);
};



class IntegrateNd : public Object{
 public:
  Object *OF;
  void (Object:: *MF)(double*,double*);
 protected:
  int N;
  double            *VarNd;
  Integrate1dnd *IntegrL;
  void FctValue(double*,double X);
 public:
  IntegrateNd(int N_,int NFunc);
  IntegrateNd(int N_,int NFunc,IntegrationDatas id[]);
  ~IntegrateNd();
  int Integrate(double* Result, Object *O,
                void(Object:: *FOjb)(double*,double *),
                double A[],double B[]);
  int OK();
  int OK(int OkArray[]);
};

#define INTEGRATEDFN(TN,TF,NFCT_)                                       \
                                                                        \
class IntegrateNd##TN : public IntegrateNd{                             \
  Object *COF;                                                          \
  TF (Object:: *CMF)(double*);                                          \
  void AFctValue(double *Res,double* Var);                              \
 public:                                                                \
  IntegrateNd##TN(int n):IntegrateNd(n,NFCT_){};                        \
  IntegrateNd##TN(int n,IntegrationDatas id[])                          \
    : IntegrateNd(n,NFCT_,id){};                                        \
  ~IntegrateNd##TN(){};                                                 \
  int OK(){return IntegrateNd::OK();};                                  \
  int OK(int OkArray[]){return IntegrateNd::OK(OkArray);};              \
  TF Integrate(Object *O,TF (Object:: *FObj)(double*),                  \
                    double*,double*);                                   \
};                                                                      \
                                                                        \
class Integrate1d##TN : public Integrate1d{                             \
  Object *COF;                                                          \
  TF (Object:: *CMF)(double);                                           \
  void AFctValue(double *Res,double Var);                               \
 public:                                                                \
  Integrate1d##TN():Integrate1d(NFCT_){};                               \
  Integrate1d##TN(IntegrationDatas id)                                  \
    : Integrate1d(NFCT_,id){};                                          \
  ~Integrate1d##TN(){};                                                 \
   int OK(){return Integrate1d::OK();};                                 \
  TF Integrate(Object *O,TF (Object:: *FObj)(double),                   \
                    double a,double b);                                 \
  TF IntegratePInf(Object *O,TF (Object:: *FObj)(double),               \
                    double a,double RenP);                              \
  TF IntegrateNInf(Object *O,TF (Object:: *FObj)(double),               \
                    double b,double RenN);                              \
  TF IntegrateNPInf(Object *O,TF (Object:: *FObj)(double),              \
                    double RenN,double RenP);                           \
};                                                                      \


#define INTEGRATEDFNMATRIX(TN,TF,NF_)        class                        \
Integrate1d##TN : public Integrate1d{                                     \
  Object *COF;                                                            \
  TF (Object:: *CMF)(double);                                             \
  void AFctValue(double *Res,double Var);                                 \
  int I;                                                                  \
  int J;                                                                  \
 public:                                                                  \
  Integrate1d##TN(int i,int j):Integrate1d(i*j*NF_){I=i;J=j;};            \
    Integrate1d##TN(int i,int j,IntegrationDatas id)                      \
    : Integrate1d(i*j*NF_,id){I=i;J=j;};                                  \
                                                                          \
  ~Integrate1d##TN(){};                                                   \
  int OK(){return Integrate1d::OK();};                                    \
  TF Integrate(Object *O,TF (Object:: *FObj)(double),double a,double b);  \
  TF IntegratePInf(Object *O,TF (Object:: *FObj)(double),                 \
                   double a, double RenP);                                \
  TF IntegrateNInf(Object *O,TF (Object:: *FObj)(double),                 \
                   double b, double RenN);                                \
  TF IntegrateNPInf(Object *O,TF (Object:: *FObj)(double),                \
                   double RenN, double RenP);                             \
};                                                                        \
                                                                          \
class IntegrateNd##TN : public IntegrateNd{                               \
  Object *COF;                                                            \
  TF (Object:: *CMF)(double*);                                            \
  void AFctValue(double *Res,double* Var);                                \
  int I;                                                                  \
  int J;                                                                  \
 public:                                                                  \
  IntegrateNd##TN(int i,int j,int n):IntegrateNd(n,i*j*NF_){I=i;J=j;};    \
  IntegrateNd##TN(int i,int j,int n,IntegrationDatas id[])                \
    : IntegrateNd(n,i*j*NF_,id){I=i;J=j;};                                \
  ~IntegrateNd##TN(){};                                                   \
  int OK(){return IntegrateNd::OK();};                                    \
  int OK(int OkArray[]){return IntegrateNd::OK(OkArray);};                \
  TF Integrate(Object *O,TF (Object:: *FObj)(double*),                    \
                    double*,double*);                                     \
};                                                                        \
                             


INTEGRATEDFN(Complex,Complex,2)
INTEGRATEDFN(Double,double,1)
INTEGRATEDFN(Vect2d,vect2d,2)
INTEGRATEDFN(Vect3d,vect3d,3)
INTEGRATEDFN(Vect2dC,vect2dc,4)
INTEGRATEDFN(Vect3dC,vect3dc,6)

//INTEGRATEDFNMATRIX(DMatrix,dmatrix,1)
//INTEGRATEDFNMATRIX(CMatrix,cmatrix,2)

#endif









