#include <Complex.h>
#include <iostream.h>
#include <stdlib.h>

#include "Object.h"
#include "Error.h"
#include "Const.h"
#include "Math.h"
#include "NAG.h"

#define DELTA 0.00001


Complex sinc(Complex arg)

{ if (Abs(arg)<DELTA)
    {
    Complex parg2=PI*PI*arg*arg;
    return 1-parg2/6+parg2*parg2/120;
    }
  else
    return sin(arg*PI)/(arg*PI);
}

double sinc(double arg)

{ if (Abs(arg)<DELTA)
    {
    double parg2=PI*PI*arg*arg;
    return 1-parg2/6+parg2*parg2/120;
    } 
  else
    return sin(arg*PI)/(arg*PI);
}


Complex Erf(Complex z)

{ int ifail;
  Complex iz;
  Complex t3;
  ifail=-1; 

  iz=Complex(0,1)*z;
  FortranCall(s15ddf)(&t3,&iz,&ifail);

  return 1-t3*exp(-pow(z,2));
}

double  Erf(double z)

{ int ifail=-1;
  
  return FortranCall(s15aef)(&z,&ifail);
}

double SinIntegral(double x)

//gives the sine integral Integrate[Sin[t]/t, {t, 0, x}]

{ int IFAIL=-1;
  double d;
  d=FortranCall(s13adf)(&x,&IFAIL);
  if(IFAIL) FatalError("SinIntegral");
  return d;
}

double CosIntegral(double x) 

//gives the cosine integral EulerGamma + Log[x] + 
//                          Integrate[(Cos[t] - 1)/t, {t, 0, x}]

{ int IFAIL=-1;
  double d;
  d=FortranCall(s13acf)(&x,&IFAIL);
  if(IFAIL) FatalError("CosIntegral");
  return d;
}

double ExpIntegralEi(double x)

// gives the exponential integral Integrate[Exp(u)/u,{u,x,Infinity}] 

{ int IFAIL=-1;
  double d;
  d=FortranCall(s13aaf)(&x,&IFAIL);
  if(IFAIL) FatalError("ExpIntegralEi");
  return d;
}


int HankelFct(int M,Complex Z,int OrderMin,int NumbOfMember,Complex *Value);
int HankelFct(int M,Complex Z,int OrderMin,int NumbOfMember,Complex *Value)

// Evaluate the Hankelfunction M of order form OrderMin to OrderMin+NumberOf
// Member-1 im Ort Z. Die relults are saved in array Value.
//
// Return 0 falls alles ist korrekt.
 
{ char SCALE='U';
  double OrderMinF=double(OrderMin);
  int NZ;                            
  int IFAIL=1;                       //Soft Fail Option: Silent exit

  FortranCall(s17dlf)(&M,&OrderMinF,&Z,&NumbOfMember,&SCALE,Value,&NZ,&IFAIL);

  if(IFAIL==0) return 0;
  for(int i=1;i<NumbOfMember;i++) Value[i]=0;
  if((IFAIL==4) && (imag(Z)>1000) && (M==1)) return 0; 
  if((IFAIL==4) && (imag(Z)<-1000) && (M==2)) return 0; 
  cerr<<"s17dlf(Hankel)Error="<<IFAIL<<" ";
  return IFAIL;
}

int ScaledHankelFct(int M,Complex Z,int OrderMin,int NumbOfMember,
                    Complex *Value);
int ScaledHankelFct(int M,Complex Z,int OrderMin,int NumbOfMember,
                    Complex *Value)

// Berechnet die Hankelfunction M von Order OrderMin bis OrderMin+NumberOf
// Member-1 im Ort Z. Die Resultaten sind im Array Value gespeichert.
//
// Return 0 falls alles ist korrekt.
 
{ char SCALE='S';
  double OrderMinF=double(OrderMin);
  int NZ;                            
  int IFAIL=1;                       //Soft Fail Option: Silent exit

  FortranCall(s17dlf)(&M,&OrderMinF,&Z,&NumbOfMember,&SCALE,Value,&NZ,&IFAIL);

  if(IFAIL==0) return 0;
  cerr<<"s17dlf(Hankel)Error="<<IFAIL<<" ";
  return IFAIL;
}

int BesselFct(Complex Z,int OrderMin,int NumbOfMember,Complex *Value)

// Berechnet die Besselfunction von Order OrderMin bis OrderMin+NumberOf
// Member-1 im Ort Z. Die Resultaten sind im Array Value gespeichert.
//
// Return 0 falls alles ist korrekt.
 
{ char SCALE='U';
  double OrderMinF=double(OrderMin);
  int NZ; 
  int IFAIL=1;                       //Soft Fail Option: Silent exit

  FortranCall(s17def)(&OrderMinF,&Z,&NumbOfMember,&SCALE,Value,&NZ,&IFAIL);

  if(IFAIL==0) return 0;
  cerr<<"s17def(Bessel)Error="<<IFAIL<<" ";
  return IFAIL;
}

int ScaledBesselFct(Complex Z,int OrderMin,int NumbOfMember,Complex *Value)

// Berechnet die Besselfunction von Order OrderMin bis OrderMin+NumberOf
// Member-1 im Ort Z. Die Resultaten sind im Array Value gespeichert.
//
// Return 0 falls alles ist korrekt.
 
{ char SCALE='S';
  double OrderMinF=double(OrderMin);
  int NZ; 
  int IFAIL=1;                       //Soft Fail Option: Silent exit

  FortranCall(s17def)(&OrderMinF,&Z,&NumbOfMember,&SCALE,Value,&NZ,&IFAIL);

  if(IFAIL==0) return 0;
  cerr<<"s17def(Bessel)Error="<<IFAIL<<" ";
  return IFAIL;
}

Complex Hankel1(int N,Complex C)

{ if(N<0) return pow(Complex(-1),N)*Hankel1(-N,C);
  Complex R;

  HankelFct(1,C,N,1,&R);
  return R;
}

Complex Hankel2(int N,Complex C)

{ if (N<0) return pow(Complex(-1),N)*Hankel2(-N,C);
  Complex R;

  HankelFct(2,C,N,1,&R);
  return R;
}

Complex Bessel(int N,Complex C)

{ if (N<0) return pow(Complex(-1),N)*Bessel(-N,C);
  Complex R;

  BesselFct(C,N,1,&R);
  return R;
}

Complex ScaledHankel1(int N,Complex C)

{ if(N<0) return pow(Complex(-1),N)*ScaledHankel1(-N,C);
  Complex R;

  ScaledHankelFct(1,C,N,1,&R);
  return R;
}

Complex ScaledHankel2(int N,Complex C)

{ if (N<0) return pow(Complex(-1),N)*ScaledHankel2(-N,C);
  Complex R;

  ScaledHankelFct(2,C,N,1,&R);
  return R;
}

Complex ScaledBessel(int N,Complex C)

{ if (N<0) return pow(Complex(-1),N)*ScaledBessel(-N,C);
  Complex R;

  ScaledBesselFct(C,N,1,&R);
  return R;
}


int HankelFct1(Complex Z,int OrderMin,int NumbOfMember,Complex *Value)
{ return HankelFct(1, Z, OrderMin, NumbOfMember, Value);}

int HankelFct2(Complex Z,int OrderMin,int NumbOfMember,Complex *Value)
{ return HankelFct(2, Z, OrderMin, NumbOfMember, Value);}

int ScaledHankelFct1(Complex Z,int OrderMin,int NumbOfMember,
                     Complex *Value)
{ return ScaledHankelFct(1, Z, OrderMin, NumbOfMember, Value);}

int ScaledHankelFct2(Complex Z,int OrderMin,int NumbOfMember,
                     Complex *Value)
{ return ScaledHankelFct(2, Z, OrderMin, NumbOfMember, Value);}



double min(double a, double b)

{ if (a<b) return a;
  else return b;
}

double max(double a, double b)

{ if (a>b) return a;
  else return b;
}

int min(int a, int b)

{ if (a<b) return a;
  else return b;
}

int max(int a, int b)

{ if (a>b) return a;
  else return b;
}

double Abs(double  v){return fabs(v);}
int    Abs(int     v){return  abs(v);}
double Abs(Complex v){return  abs(v);}


double RandomNormalDistribution(double StandartDeviation)

{ double a=0;
  return FortranCall(g05ddf)(a,StandartDeviation);
}

double RandomNormalDistribution()
 
{ return RandomNormalDistribution(1);
}





#if !defined(__GNUC__)

double pow(double x,int n)
{ double res;
  switch (n)
    {
    case (-3) :{res=1/x/x/x; break;}
    case (-2) :{res=1/x/x;   break;}
    case (-1) :{res=1/x;     break;}
    case (0) : {res=1;       break;}
    case (1) : {res=x;       break;}
    case (2) : {res=x*x;     break;}
    case (3) : {res=x*x*x;   break;}
    case (4) : {res=x*x*x*x; break;}

    default : res=pow(x,(double)n);
    }
  return res;
}

#endif




