/***************************************************************************/
/*                                                                         */
/*   Untersuchung eines neuen Verfahrens zur Berechnung von Streufeldern.  */
/*                   The Mesaured Equation of Invariance                   */
/*                                                                         */
/***************************************************************************/
/*   File : Vect.h                                                         */
/*                                                                         */
/*   Geschrieben : 04.07.95 Nicolas Piller                                 */
/*   Aenderungen : ...      ...                                            */
/*                                                                         */
/***************************************************************************/

#if ! defined(_VECT_)
#define _VECT_

#include <Complex.h>
#include <iostream.h>
#include "Const.h"

class vect2d;
class vect2dc;
class vect3d;
class vect3dc;

typedef enum{MatlabVectorOutputMethod,
             MMPVectorOutputMethod,
             MathematicaVectorOutputMethod  } VectorOutputMethodT;

typedef enum{MatlabVectorInputMethod,
             MMPVectorInputMethod,
             MathematicaVectorInputMethod  } VectorInputMethodT;

extern VectorOutputMethodT VectorOutputMethod;
extern VectorInputMethodT VectorInputMethod;

//**************************************vect2d

class vect2d {
public:
  double x;
  double y;

  vect2d();
  vect2d(double X,double Y);
  friend vect2d MakeVect2D(vect3d);
  ~vect2d();
  friend double norm(vect2d);  
  friend double Abs(vect2d);
  friend double arg(vect2d v){return atan2(v.y,v.x);}; 
  friend vect2d unit(vect2d);

  friend vect2d operator+(vect2d,vect2d);
  friend vect2d operator-(vect2d,vect2d);
  friend vect2d operator-(vect2d);
  friend double operator*(vect2d,vect2d);
  friend vect2d operator*(vect2d,double);
  friend vect2d operator*(double,vect2d);
  friend vect2d operator/(vect2d,double);
  
  vect2d& operator+=(vect2d);
  vect2d& operator-=(vect2d);

  friend int operator==(vect2d,vect2d);
  friend int operator!=(vect2d,vect2d);
};

//**************************************vect2dc

class vect2dc {
public:
  Complex x;
  Complex y;

  vect2dc();
  vect2dc(Complex X,Complex Y);
  vect2dc(vect2d Re, vect2d Im);
  friend vect2dc MakeVect2D(vect3dc);
  ~vect2dc();

  friend double  norm(vect2dc);  
  friend double  Abs(vect2dc);
  friend vect2dc unit(vect2dc);
  friend vect2dc conj(vect2dc);
  friend vect2d  real(vect2dc);
  friend vect2d  imag(vect2dc);

  friend vect2dc  operator+(vect2dc,vect2dc);
  friend vect2dc  operator+(vect2dc,vect2d );
  friend vect2dc  operator+(vect2d ,vect2dc);
  friend vect2dc  operator-(vect2dc,vect2dc);
  friend vect2dc  operator-(vect2d ,vect2dc);
  friend vect2dc  operator-(vect2dc,vect2d );

  friend vect2dc  operator-(vect2dc);

  friend Complex operator*(vect2dc,vect2dc);
  friend Complex operator*(vect2d ,vect2dc);
  friend Complex operator*(vect2dc,vect2d );
  friend vect2dc  operator*(double ,vect2dc);
  friend vect2dc  operator*(Complex,vect2dc);
  friend vect2dc  operator*(Complex,vect2d );

  friend vect2dc  operator*(vect2dc,double );
  friend vect2dc  operator*(vect2d ,Complex);
  friend vect2dc  operator*(vect2dc,Complex);
  friend vect2dc  operator/(vect2dc,double );
  friend vect2dc  operator/(vect2d ,Complex);
  friend vect2dc  operator/(vect2dc,Complex);

  vect2dc& operator+=(vect2dc);
  vect2dc& operator-=(vect2dc);
  vect2dc& operator+=(vect2d);
  vect2dc& operator-=(vect2d);

  friend int operator==(vect2dc,vect2dc);
  friend int operator!=(vect2dc,vect2dc);
};
  
//**************************************vect3d

class vect3d {
public:
  double x;
  double y;
  double z;

  vect3d();
  vect3d(double X,double Y, double Z);
  vect3d(vect2d);
  ~vect3d();
  friend double norm(vect3d);  
  friend double Abs(vect3d);
  friend vect3d unit(vect3d);

  friend vect3d operator+(vect3d,vect3d);
  friend vect3d operator-(vect3d,vect3d);
  friend vect3d operator-(vect3d);
  friend double operator*(vect3d,vect3d);
  friend vect3d operator*(vect3d,double);
  friend vect3d operator*(double,vect3d);
  friend vect3d operator/(vect3d,double);
  
  friend vect3d operator^(vect3d,vect3d);

  vect3d& operator+=(vect3d);
  vect3d& operator-=(vect3d);

  friend int operator==(vect3d,vect3d);
  friend int operator!=(vect3d,vect3d);
};

//**************************************vect3dc

class vect3dc {
public:
  Complex x;
  Complex y;
  Complex z;
 
  vect3dc();
  vect3dc(Complex X,Complex Y, Complex Z);
  vect3dc(vect3d Re, vect3d Im);
  vect3dc(vect2dc);
  ~vect3dc();

  friend double  norm(vect3dc);  
  friend double  Abs(vect3dc);
  friend vect3dc unit(vect3dc);
  friend vect3dc conj(vect3dc);
  friend vect3d real(vect3dc v);

  friend vect3d  imag(vect3dc v);

  friend vect3dc  operator+(vect3dc,vect3dc V3);
  friend vect3dc  operator+(vect3dc,vect3d  V3);
  friend vect3dc  operator+(vect3d ,vect3dc V3);
  friend vect3dc  operator-(vect3dc,vect3dc V3);
  friend vect3dc  operator-(vect3d ,vect3dc V3);
  friend vect3dc  operator-(vect3dc,vect3d  V3);

  friend vect3dc  operator-(vect3dc);

  friend Complex operator*(vect3dc,vect3dc V3);
  friend Complex operator*(vect3d ,vect3dc V3);
  friend Complex operator*(vect3dc,vect3d  V3);
  friend vect3dc  operator*(double ,vect3dc V3);
  friend vect3dc  operator*(Complex,vect3dc V3);
  friend vect3dc  operator*(Complex,vect3d  V3);

  friend vect3dc  operator*(vect3dc,double  V3);
  friend vect3dc  operator*(vect3d ,Complex V3);
  friend vect3dc  operator*(vect3dc,Complex V3);
  friend vect3dc  operator/(vect3dc,double  V3);
  friend vect3dc  operator/(vect3d ,Complex V3);
  friend vect3dc  operator/(vect3dc,Complex V3);

  friend vect3dc operator^(vect3dc,vect3dc);
  friend vect3dc operator^(vect3d ,vect3dc);
  friend vect3dc operator^(vect3dc,vect3d );

  vect3dc& operator+=(vect3dc);
  vect3dc& operator-=(vect3dc);
  vect3dc& operator+=(vect3d);
  vect3dc& operator-=(vect3d);

  friend int operator==(vect3dc,vect3dc);
  friend int operator!=(vect3dc,vect3dc);

  int IsNormalToXYPlane(double accrel);
  int IsParallelToXYPlane(double accrel);
  int IsNormalToXYPlane();
  int IsParallelToXYPlane();
  void MakeNormalToXYPlane();
  void MakeParallelToXYPlane();

};
  
ostream& operator<<(ostream& o,const vect2dc&);
ostream& operator<<(ostream& o,const vect2d&);
ostream& operator<<(ostream& o,const vect3dc&);
ostream& operator<<(ostream& o,const vect3d&);

istream& operator>>(istream& o,vect2dc&);
istream& operator>>(istream& o,vect2d&);
istream& operator>>(istream& o,vect3dc&);
istream& operator>>(istream& o,vect3d&);

ostream& mmpwrite(ostream& o,const Complex& C); 
ostream& mmpwrite(ostream& o,const vect2d& C);
ostream& mmpwrite(ostream& o,const vect3d& C);
ostream& mmpwrite(ostream& o,const vect2dc& C);
ostream& mmpwrite(ostream& o,const vect3dc& C);



#endif

